import os
from direct.task import Task
from direct.actor import Actor
from panda3d.core import AmbientLight,DirectionalLight,PointLight
from panda3d.core import Point3,Vec3,Vec4,Mat4
from panda3d.core import ConfigVariableString
from pandac.PandaModules import Filename
from numpy import *
from pylab import *

# Local fix to setup configuration variables for remote graphics
if(os.uname()[1] == "u3.csc.kth.se"):
  display_conf = ConfigVariableString('load-display')
  print display_conf.getValue()
  display_conf.setValue('tinydisplay')
  print display_conf.getValue()
  window_conf = ConfigVariableString('win-size')
  print window_conf.getValue()
  window_conf.setValue('800 600')
  print window_conf.getValue()
  audio_conf = ConfigVariableString('audio-library-name')
  print audio_conf.getValue()
  audio_conf.setValue('none')
  print audio_conf.getValue()

import direct.directbase.DirectStart

# Visualization model
class myGame:
  def __init__(self):
    self.running = True
    self.kick = 1.0
    self.N = 1000000
    self.tarr = zeros(self.N)
    self.xarr = zeros(self.N)
    self.varr = zeros(self.N)
    self.farr = zeros(self.N)


# Quit event handler
def quit(g):
  print "Quitting"
  g.running = False

# Kick event handler
def kick(g):
  print "Kick"
  if(g.kick == 1.0):
    g.kick = 1.2
  else:
    g.kick = 1.0

game = myGame()

width = 9.0
height = 9.0

# Load the ball model
def init_spheres(ball, radius, M):
  for i in range(0, M):
    ball.append(loader.loadModel("data/ball"))
    ball[i].setScale(radius, radius, radius)
    ball[i].reparentTo(render)

base.accept("escape", quit, [game])
base.accept("q", quit, [game])
base.accept("k", kick, [game])

#Task to move the camera
def spinCameraTask(task):
  camera.setPos(5, 0, 60.0)
  camera.lookAt(Point3(0,0,0), Vec3(0,1,0))
  mat=Mat4(camera.getMat())
  mat.invertInPlace()
  base.mouseInterfaceNode.setMat(mat)
  base.enableMouse()
  return Task.done

#Task to move the ball
def moveBallTask(task):
  for i in range(0, game.M):
    game.ball[i].setPos(game.x[i][0], game.x[i][1], 0.0)
  return Task.cont

def update_graphics(counter, x, dt):
  if(counter % int(1.0/dt/10.0) == 0):
    game.x = x
    taskMgr.step()
    #base.screenshot("screenshot")

def init_graphics(x, radius, M):
  game.radius = radius
  game.M = M

  # Load the environment model
  game.environ = loader.loadModel("data/board")
  game.environ.reparentTo(render)
  game.environ.setScale(9.0 / 0.8, 9.0 / 0.8, 9.0 / 0.8)
  game.environ.setPos(0, 0, -5)

  game.ball = []
  init_spheres(game.ball, radius, M)

  plight = PointLight('Lamp')
  plight.setColor(Vec4(.4, .4, .35, 1))
  pnode = game.environ.attachNewNode(plight)
  pnode.setPos(0.7, 0, 0.5)
  for i in range(0, M):
    game.ball[i].setLight(pnode)
    game.environ.setLight(pnode)

    game.x = x
    taskMgr.add(moveBallTask, "moveSphereTask")
    taskMgr.add(spinCameraTask, "SpinCameraTask")
 

def particle_plot(counter):
  # Plot constructed functions
  subplot(311)
  plot(game.tarr[:counter], game.farr[:counter])
  xlabel('t')
  ylabel('f')
  subplot(312)
  plot(game.tarr[:counter], game.varr[:counter])
  xlabel('t')
  ylabel('v')
  subplot(313)
  plot(game.tarr[:counter], game.xarr[:counter])
  xlabel('t')
  ylabel('x')
  show()
  os._exit(0)

def store(counter, t, x, v, fval):
  """Store values of one component for later plotting"""
  game.tarr[counter] = t
  game.xarr[counter] = x[0][1]
  game.varr[counter] = v[0][1]
  game.farr[counter] = fval[1]
