from bisect import *
from pylab import *

set_printoptions(3)

def timestep(f, t0, u0, k, method):
    """Take one time step for the initial value problem (IVP).

    f: function describing IVP u' = f(t, u)
    t: float for time value f(t, u)
    u0: float for initial value
    k: float for time-step value
    method: string naming the time-step method
    
    Returns float u = u0 + k * f(t, u0)

    Reference test:
    def f1(t, u): return t*t
    timestep(f1, 0.1, 1.0, 0.1, "Euler") => 1.001
    def f2(t, u): return -u
    timestep(f2, 0.1, 1.0, 0.1, "Euler") => 0.9
    """

    if(method == "Euler"):
        u = u0 + k * f(t0, u0)
    elif(method == "Midpoint"):
        t = t0 + k
        # Fill in missing implementation
        u = 0
    else:
        print "wrong method"
        exit(0)

    return u

def solve(f, I, u0, k, method):
    """Solve the IVP f by time-stepping

    f: function describing IVP u' = f(t, u)
    I: float array for time interval
    u0: float for initial value
    k: float for time-step value
    method: string naming the time-step method
    
    Returns array of arrays [tarray, uarray] for time values and u values

    Reference test:
    def f(t, u): return -u
    [tarray, uarray] = solve(f, [0, 4], 1, 0.01, "Euler")
    plot(tarray, uarray)
    Expecting the function u(t) = exp(-t) and uarray[0, 2] == 0.98
    """

    t0 = I[0]
    T = I[1]
    tarray = linspace(t0, T, ceil((T-t0)/k) + 1)
    uarray = zeros([size(u0), tarray.size])
    uarray[:, 0] = u0
    u = u0
    i = 1
    # Fill in timestepping implementation

    return [tarray, uarray]

def piecewise_linear_adapter(tarray, uarray):
    """Returns a piecewise linear function given the nodes and node values."""
    def u_adapted(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[:, i0]
        u1 = uarray[:, i1]

        w0 = (t1 - t) / (t1 - t0)
        w1 = (t - t0) / (t1 - t0)

        ut = w0*u0 + w1*u1
        return ut

    return u_adapted

def derivative_adapter(tarray, uarray):
    """Returns the derivative of a piecewise linear function."""
    def derivative(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[:, i0]
        u1 = uarray[:, i1]

        k = t1 - t0
        dudt = (u1 - u0) / (t1 - t0)

        ut = dudt
        return ut

    return derivative


def maximum_residual(tarray, u, dudt, f):
    """Return the maximum residual by computing the maximum residual
    in each interval (one of the end points)"""
    R = 0.0
    for t in tarray:
        R=0.0 # Just for making the program work!!
        # Compute the residual
        
    return R


